from typing import IO, List
from ludvig import Finding, Severity
from ludvig.config import Config
from ._common import BaseScanner
from ludvig.vulndb import VulnDb
from ludvig.parsers.dotnet_deps import parse_dotnet_deps, parse_dotnet_proj


class VulnerabilityScanner(BaseScanner):
    def __init__(self, vuln_db: VulnDb, config: Config) -> None:
        super().__init__(False, config)
        self.__db = vuln_db
        self.__parsers = [parse_dotnet_deps, parse_dotnet_proj]

    def scan_file_data(
        self, file_data: IO[bytes], file_name: str, severity_level: Severity, **kwargs
    ) -> List[Finding]:
        findings = []
        for parser in self.__parsers:
            packages = parser(file_data.read())
            for package in packages:
                results = self.__db.query_advisory(
                    package_name=package["name"],
                    ecosystem=package["ecosystem"],
                    version=package["version"],
                )
                for result in results:
                    findings.append(
                        Finding.from_vuln_advisory(
                            result, package["version"], file_name
                        )
                    )
        return findings

    def accepted_files(self) -> List[str]:
        return ["*.deps.json", "*.csproj"]



from ludvig.rules import download_rules
from ludvig.config import RuleSetSource, get_config


def download():
    """
    Downloads and compiles the configured rule sets.
    """
    current_config = get_config()
    download_rules(current_config)


def add_repo(name: str, category: str, url: str):
    """
    Adds a custom repository of YARA rules (needs to be in tar-gzipped format)
    """
    
    current_config = get_config()
    current_config.rule_sources.append(RuleSetSource(name, category, url))
    current_config.save()



from defusedxml.ElementTree import fromstring
import json


def parse_dotnet_proj(project_file: str):
    import xml

    try:
        tree = fromstring(project_file)
    except xml.etree.ElementTree.ParseError:
        return []
    refs = tree.findall("*/PackageReference")
    return [
        {
            "ecosystem": "nuget",
            "name": r.attrib["Include"],
            "version": r.attrib["Version"] if "Version" in r.attrib else "0",
        }
        for r in refs
    ]


def parse_dotnet_deps(deps_file: str):
    try:
        deps_data = json.loads(deps_file)
    except json.JSONDecodeError:
        return []
    libs = deps_data["libraries"]
    return [
        {
            "ecosystem": "nuget",
            "name": lib[: lib.find("/")],
            "version": lib[lib.find("/") + 1 :],
        }
        for lib in libs
        if libs[lib]["type"] == "package"
    ]



from unittest import TestCase
import os
from ludvig.sbom.dotnet.dependencies import DotnetDependenciesParser
from cyclonedx.model.bom import Bom
from cyclonedx.output import OutputFormat, get_instance as get_output
from ludvig.parsers.dotnet_deps import parse_dotnet_deps, parse_dotnet_proj


class TestDotnetDependenciesParser(TestCase):
    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)

    def test_simple(self) -> None:
        with open(
            os.path.join(os.path.dirname(__file__), "fixtures/dotnet.deps-example.json")
        ) as r:
            parser = DotnetDependenciesParser(dependency_content=r.read())
            bom = Bom.from_parser(parser)
            v = get_output(bom=bom, output_format=OutputFormat.JSON)

    def test_parse_dotnet_csproj(self) -> None:
        proj_file = os.path.join(os.path.dirname(__file__), "fixtures/dotnet.csproj")
        with open(proj_file) as f:
            packages = parse_dotnet_proj(f.read())
        self.assertEqual(len(packages), 1)
        self.assertEqual(packages[0]["name"], "Newtonsoft.Json")
        self.assertEqual(packages[0]["version"], "13.0.2")

    def test_parse_dotnet_deps(self) -> None:
        deps_file = os.path.join(
            os.path.dirname(__file__), "fixtures/dotnet.deps-example.json"
        )
        with open(deps_file) as f:
            packages = parse_dotnet_deps(f.read())
            self.assertEqual(len(packages), 2)
            self.assertEqual(packages[0]["name"], "IdentityModel")
            self.assertEqual(packages[0]["version"], "6.0.0")
